<html>
<head>
  <title>{{ title }}</title>
  <meta charset="utf-8">
  <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css" integrity="sha384-BVYiiSIFeK1dGmJRAkycuHAHRg32OmUcww7on3RYdg4Va+PmSTsz/K68vbdEjh4u" crossorigin="anonymous">
  <link rel="stylesheet" href="https://cdn.jsdelivr.net/hint.css/2.4.0/hint.min.css" integrity="sha256-8tIlfXMO4Ui7kMz0olkEGYX8yNma7oowG+SZwnV+tYs=" crossorigin="anonymous">
  <style type="text/css">
    h1, p {
      text-align: center;
    }
  </style>
</head>
<body>
<div class="container-fluid">
{% if debug %}
  <div class="alert alert-danger" role="alert" style="text-align: center;">
    <b>DEBUG</b>
  </div>
{% endif %}
<h1 class="page-header">{{ title }}</h1>
{% for headline, list in objs %}
  {% if list|length > 0 %}
  {% set headline_id = headline|lower|replace(' ', '_') %}
  <h2>{{ headline }}</h2>
  <ul class="list-group" id="{{ headline_id }}">
    {% for client, value in list %}
    {% set id = headline_id + "." + client.nick %}
    <li id="{{ id }}" onclick="window.location = '#{{ id }}'" class="list-group-item{{ ' list-group-item-success' if client.connected else loop.cycle('" style="background-color: #eee;', '') }}">
      <span {% if not client.connected %}class="hint--right" data-hint="{{ client.last_seen|frmttime }}"{% endif %}>{{ client.nick }}{{ " (" + client.identifier + ")" if debug }}</span>
      <span class="badge">{{ value }}</span>
    </li>
    {% endfor %}
  </ul>
  {% endif %}
{% endfor %}
<small>Generated by <a href="https://github.com/Thor77/TeamspeakStats">TeamspeakStats</a> at {{ creation_time|frmttime }}</small>
</div>
</body>
</html>
